<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="color-scheme" content="light dark" />
    <meta name="generator" content="ToDo Merger {{ app_version }}">
    <title>ToDo Merger</title>
    <!-- CSS -->
    <link rel="stylesheet" href="{{ url_for('static', filename='css/pico.min.css') }}" />
    <link rel="stylesheet" href="{{ url_for('static', filename='fontawesome/fontawesome-all.min.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/custom.scss.css') }}" />
    <!-- Favicon -->
    <link rel="apple-touch-icon" sizes="180x180" href="{{ url_for('static', filename='img/apple-touch-icon.png') }}">
    <link rel="icon" type="image/png" sizes="32x32" href="{{ url_for('static', filename='img/favicon-32x32.png') }}">
    <link rel="icon" type="image/png" sizes="16x16" href="{{ url_for('static', filename='img/favicon-16x16.png') }}">
    <link rel="manifest" href="{{ url_for('static', filename='img/site.webmanifest') }}">
    <link rel="mask-icon" href="{{ url_for('static', filename='img/safari-pinned-tab.svg') }}" color="#5bbad5">
    <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.ico') }}">
    <meta name="msapplication-TileColor" content="#2b5797">
    <meta name="msapplication-config" content="{{ url_for('static', filename='img/browserconfig.xml') }}">
    <meta name="theme-color" content="#ffffff">
  </head>
  <body>
    <main class="container">
      {% block content %}
      {% endblock %}
    </main>
  </body>
</html>
